// Based on examples
// - https://crates.io/crates/gtk4
// - https://github.com/gtk-rs/gtk4-rs/blob/main/examples/clock/main.rs
// - https://github.com/gtk-rs/gtk4-rs/blob/main/examples/clipboard/main.rs
// - https://github.com/kashifsoofi/blog-code-samples/blob/gtk4-c-counter-app/gtk4-rust-counter-app/src/main.rs

use chrono::Local;
use std::cell::Cell;
use std::rc::Rc;

use gtk::gio::{Notification, NotificationPriority};
use gtk::glib;
use gtk::glib::clone;
use gtk::prelude::*;
use gtk4 as gtk;

const APP_ID: &str = "de.privacy1st.pdt";

const DUR_MIN: f64 = 1.0;
const INTERVAL_SEC: u32 = 1;

fn main() -> glib::ExitCode {
    let app = gtk::Application::builder().application_id(APP_ID).build();
    app.connect_activate(move |app| build_ui(app));
    app.run()
}

fn build_ui(app: &gtk::Application) {
    let window = gtk::ApplicationWindow::builder()
        .application(app)
        .title("PleaseDisturbTimer")
        .default_width(350)
        .default_height(70)
        .build();

    let container = gtk::Box::builder()
        .orientation(gtk::Orientation::Vertical)
        .margin_top(24)
        .margin_bottom(24)
        .margin_start(24)
        .margin_end(24)
        .halign(gtk::Align::Center)
        .valign(gtk::Align::Center)
        .spacing(24)
        .build();

    // How to change variable using GTK4 Button? https://stackoverflow.com/a/76657798/6334421
    let running = Rc::new(Cell::new(false));

    let button_start = gtk::Button::with_label("Start");
    button_start.connect_clicked(clone!(@strong running => move |_| {
        running.set(true);
        eprintln!("Timer started.");
    }));
    container.append(&button_start);

    let button_stop = gtk::Button::with_label("Stop");
    button_stop.connect_clicked(clone!(@strong running => move |_| {
            running.set(false);
            eprintln!("Timer stopped.");
    }));
    container.append(&button_stop);

    let mut progress: f64 = 0.0;
    let progress_bar = gtk::ProgressBar::new();
    progress_bar.set_fraction(progress);
    container.append(&progress_bar);

    let time = current_time();
    let label = gtk::Label::default();
    label.set_text(&time);
    // label.set_halign(gtk::Align::Center);
    container.append(&label);

    window.set_child(Some(&container));
    window.present();

    let tick = clone!(@strong app => move || {
        if running.get() {
            progress += ((INTERVAL_SEC as f64) / 60.0) / DUR_MIN;
            progress_bar.set_fraction(progress);
        }

        if progress > 1.0 {
            eprintln!("Time over.");

            // TODO: Notification does not show up
            let notification = Notification::new("Alert");
            notification.set_body(Some("Time is over"));
            notification.set_priority(NotificationPriority::Urgent);
            app.send_notification(Some(APP_ID), &notification);
        }

        let time = current_time();
        label.set_text(&time);
        // we could return glib::ControlFlow::Break to stop our clock after this tick
        glib::ControlFlow::Continue
    });

    // executes the closure once every `INTERVAL_SEC` seconds
    glib::timeout_add_seconds_local(INTERVAL_SEC, tick);
}

fn current_time() -> String {
    format!("{}", Local::now().format("%Y-%m-%d %H:%M:%S"))
}
